<?php

namespace Drupal\mediasync\Controller;

use Drupal\media\Entity\Media;
use Drupal\file\Entity\File;
use \RecursiveIteratorIterator;
use \RecursiveDirectoryIterator;

/**
 * Sync the file system with the media entities.
 */
class SyncController {

  /**
   * File mapping.
   *
   * @var array
   */
  private $fileMapping = [];

  /**
   * User id of the new media owner.
   *
   * @var int
   */
  private $uid;

  /**
   * Starts the sync of media entities with the filesystem.
   */
  public function sync() {

    $config = \Drupal::config('mediasync.settings');

    $this->uid = $config->get('user');

    $this->createMappingTable($config->get('type'));

    $query = \Drupal::entityQuery('media');
    $res = $query->execute();
    $existingMediaFiles = [];

    foreach ($res as $mid) {
      $media = Media::load($mid);
      $mediaSource = $media->getSource();
      $mediaSourceValue = $mediaSource->getSourceFieldValue($media);
      $fid = $media->getSource()->getSourceFieldValue($media);
      $file = File::load($fid);
      $uri = $file->getFileUri();
      $existingMediaFiles[] = $uri;

    }

    $directories = explode(PHP_EOL, $config->get('folder'));

    $systemFiles = [];

    foreach ($directories as $dir) {
      if (is_dir($dir)) {
        $systemFiles = array_merge($systemFiles, $this->getDirContents($dir));
      }
    }

    $diff = array_diff($systemFiles, $existingMediaFiles);
    $this->createMedia($diff);
  }

  /**
   * Create an array with all files of the specified folders.
   */
  private function getDirContents($dir) {

    $rii = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($dir));
    $files = [];
    foreach ($rii as $file) {
      if ($file->isDir()) {
        continue;
      }
      $files[] = $file->getPathname();
    }
    return $files;
  }

  /**
   * Return array whith keys as mapping target, values as supported extensions.
   */
  private function createMappingTable($configValues) {

    $mappingValues = explode(PHP_EOL, $configValues);
    foreach ($mappingValues as $value) {
      $values = explode("=>", $value);
      $fileTypes = explode(", ", $values[0]);
      $field = str_replace(" ", "", $values[2]);
      $field = str_replace("\t", "", $field);
      $field = str_replace("\r", "", $field);
      $field = str_replace("\n", "", $field);
      $key = str_replace(" ", "", $values[1]) . "." . $field;
      $this->fileMapping[$key] = $fileTypes;
    }
  }

  /**
   * Create the media entities.
   */
  private function createMedia($mediaFiles) {
    foreach ($mediaFiles as $filePath) {

      $fileType = end(explode(".", $filePath));

      $fileName = end(explode("/", $filePath));
      $fileName = str_replace(".", "", $fileName);
      $fileName = str_replace($fileType, "", $fileName);

      $extensionInfo = explode(".", $this->getMediaType($fileType));
      $bundle = $extensionInfo[0];
      $field_name = $extensionInfo[1];

      if ($bundle !== NULL) {

        $file = File::create([
          'uid' => $this->uid,
          'filename' => $fileName . "." . $fileType,
          'uri' => $filePath,
          'status' => 1,
        ]);

        $file->save();
        $media = Media::create([
          'bundle' => $bundle,
          'uid' => $this->uid,
          $field_name => [
            'target_id' => $file->id(),
          ],
        ]);

        $media->setName($fileName)->setPublished(TRUE)->save();
      }
    }
  }

  /**
   * Based on the file extension and the mapping table return the media type.
   */
  public function getMediaType($fileType) {
    foreach ($this->fileMapping as $key => $types) {
      if (in_array($fileType, $types)) {
        return $key;
      }
    }
    return NULL;
  }

}
